@@ -1,3 +1,4 @@
+import os
 import random
 import sys
 import tempfile
@@ -15,8 +16,8 @@ from rich.text import Text
 
 from scalene import sparkline
 from scalene.scalene_json import ScaleneJSON
-from scalene.scalene_statistics import Filename, LineNumber, ScaleneStatistics
 from scalene.scalene_leak_analysis import ScaleneLeakAnalysis
+from scalene.scalene_statistics import Filename, LineNumber, ScaleneStatistics
 from scalene.syntaxline import SyntaxLine
 
 
@@ -34,6 +35,15 @@ class ScaleneOutput:
     # Color for highlighted text (over the threshold of CPU time)
     highlight_color = "bold red"
 
+    # Color for memory consumption
+    memory_color = "dark_green"
+
+    # Color for GPU activity
+    gpu_color = "yellow4"
+
+    # Color for copy volume
+    copy_volume_color = "yellow4"
+
     def __init__(self) -> None:
         # where we write profile info
         self.output_file = ""
@@ -49,16 +59,16 @@ class ScaleneOutput:
     def output_top_memory(
         self, title: str, console: Console, mallocs: Dict[LineNumber, float]
     ) -> None:
-        # Print the top N lines by memory consumption, as long
-        # as they are above some threshold MB in size.
-        print_top_mallocs_count = 5
-        print_top_mallocs_threshold_mb = 1
-        if len(mallocs) > 0:
+        if mallocs:
             printed_header = False
             number = 1
-            for malloc_lineno in mallocs:
+            # Print the top N lines by memory consumption, as long
+            # as they are above some threshold MB in size.
+            print_top_mallocs_count = 5
+            print_top_mallocs_threshold_mb = 1
+            for malloc_lineno, value in mallocs.items():
                 # Don't print lines with less than the threshold MB allocated.
-                if mallocs[malloc_lineno] <= print_top_mallocs_threshold_mb:
+                if value <= print_top_mallocs_threshold_mb:
                     break
                 # Only print the top N.
                 if number > print_top_mallocs_count:
@@ -68,7 +78,7 @@ class ScaleneOutput:
                     console.print(title)
                     printed_header = True
                 output_str = f"({str(number)}) {malloc_lineno:5.0f}: {(mallocs[malloc_lineno]):5.0f} MB"
-                console.print(Markdown(output_str, style="dark_green"))
+                console.print(Markdown(output_str, style=self.memory_color))
                 number += 1
 
     def output_profile_line(
@@ -132,20 +142,6 @@ class ScaleneOutput:
                 else f"{(obj['n_peak_mb'] / 1024):5.2f}G"
             )
 
-        n_usage_fraction_str: str = (
-            ""
-            if obj["n_usage_fraction"] < 0.01
-            else f"{(100 * obj['n_usage_fraction']):4.0f}%"
-        )
-        n_python_fraction_str: str = (
-            ""
-            if obj["n_python_fraction"] < 0.01
-            else f"{(obj['n_python_fraction'] * 100):4.0f}%"
-        )
-        n_copy_mb_s_str: str = (
-            "" if obj["n_copy_mb_s"] < 0.5 else f"{obj['n_copy_mb_s']:6.0f}"
-        )
-
         # Only report utilization where there is more than 1% CPU total usage.
         sys_str: str = (
             "" if obj["n_sys_percent"] < 1 else f"{obj['n_sys_percent']:4.0f}%"
@@ -171,11 +167,11 @@ class ScaleneOutput:
                 )
             else:
                 random_samples = samples
-            sparkline_samples = []
-            for i in range(0, len(random_samples)):
-                sparkline_samples.append(
-                    random_samples[i][1] * obj["n_usage_fraction"]
-                )
+            sparkline_samples = [
+                random_samples[i][1] * obj["n_usage_fraction"]
+                for i in range(len(random_samples))
+            ]
+
             if random_samples:
                 _, _, spark_str = sparkline.generate(
                     sparkline_samples, 0, stats.max_footprint
@@ -187,6 +183,11 @@ class ScaleneOutput:
             nufs: Any = ""
             ngpus: Any = ""
 
+            n_usage_fraction_str: str = (
+                ""
+                if obj["n_usage_fraction"] < 0.01
+                else f"{(100 * obj['n_usage_fraction']):4.0f}%"
+            )
             if (
                 obj["n_usage_fraction"] >= self.highlight_percentage
                 or (
@@ -214,36 +215,45 @@ class ScaleneOutput:
                 ngpus = n_gpu_percent_str
                 nufs = spark_str + n_usage_fraction_str
 
-            if not reduced_profile or ncpps + ncpcs + nufs + ngpus:
-                if self.gpu:
-                    tbl.add_row(
-                        print_line_no,
-                        ncpps,  # n_cpu_percent_python_str,
-                        ncpcs,  # n_cpu_percent_c_str,
-                        sys_str,
-                        ngpus,
-                        n_python_fraction_str,
-                        n_growth_mem_str,
-                        nufs,  # spark_str + n_usage_fraction_str,
-                        n_copy_mb_s_str,
-                        line,
-                    )
-                else:
-                    tbl.add_row(
-                        print_line_no,
-                        ncpps,  # n_cpu_percent_python_str,
-                        ncpcs,  # n_cpu_percent_c_str,
-                        sys_str,
-                        n_python_fraction_str,
-                        n_growth_mem_str,
-                        nufs,  # spark_str + n_usage_fraction_str,
-                        n_copy_mb_s_str,
-                        line,
-                    )
-                return True
-            else:
+            if reduced_profile and not ncpps + ncpcs + nufs + ngpus:
                 return False
 
+            n_python_fraction_str: str = (
+                ""
+                if obj["n_python_fraction"] < 0.01
+                else f"{(obj['n_python_fraction'] * 100):4.0f}%"
+            )
+            n_copy_mb_s_str: str = (
+                ""
+                if obj["n_copy_mb_s"] < 0.5
+                else f"{obj['n_copy_mb_s']:6.0f}"
+            )
+
+            if self.gpu:
+                tbl.add_row(
+                    print_line_no,
+                    ncpps,  # n_cpu_percent_python_str,
+                    ncpcs,  # n_cpu_percent_c_str,
+                    sys_str,
+                    ngpus,
+                    n_python_fraction_str,
+                    n_growth_mem_str,
+                    nufs,  # spark_str + n_usage_fraction_str,
+                    n_copy_mb_s_str,
+                    line,
+                )
+            else:
+                tbl.add_row(
+                    print_line_no,
+                    ncpps,  # n_cpu_percent_python_str,
+                    ncpcs,  # n_cpu_percent_c_str,
+                    sys_str,
+                    n_python_fraction_str,
+                    n_growth_mem_str,
+                    nufs,  # spark_str + n_usage_fraction_str,
+                    n_copy_mb_s_str,
+                    line,
+                )
         else:
 
             # Red highlight
@@ -266,28 +276,27 @@ class ScaleneOutput:
                 ncpcs = n_cpu_percent_c_str
                 ngpus = n_gpu_percent_str
 
-            if not reduced_profile or ncpps + ncpcs + ngpus:
-                if self.gpu:
-                    tbl.add_row(
-                        print_line_no,
-                        ncpps,  # n_cpu_percent_python_str,
-                        ncpcs,  # n_cpu_percent_c_str,
-                        sys_str,
-                        ngpus,  # n_gpu_percent_str
-                        line,
-                    )
-                else:
-                    tbl.add_row(
-                        print_line_no,
-                        ncpps,  # n_cpu_percent_python_str,
-                        ncpcs,  # n_cpu_percent_c_str,
-                        sys_str,
-                        line,
-                    )
-
-                return True
-            else:
+            if reduced_profile and not ncpps + ncpcs + ngpus:
                 return False
+            if self.gpu:
+                tbl.add_row(
+                    print_line_no,
+                    ncpps,  # n_cpu_percent_python_str,
+                    ncpcs,  # n_cpu_percent_c_str,
+                    sys_str,
+                    ngpus,  # n_gpu_percent_str
+                    line,
+                )
+            else:
+                tbl.add_row(
+                    print_line_no,
+                    ncpps,  # n_cpu_percent_python_str,
+                    ncpcs,  # n_cpu_percent_c_str,
+                    sys_str,
+                    line,
+                )
+
+        return True
 
     def output_profiles(
         self,
@@ -296,6 +305,7 @@ class ScaleneOutput:
         pid: int,
         profile_this_code: Callable[[Filename, LineNumber], bool],
         python_alias_dir: Path,
+        program_path: Path,
         profile_memory: bool = True,
         reduced_profile: bool = False,
     ) -> bool:
@@ -357,7 +367,7 @@ class ScaleneOutput:
                 if stats.max_footprint > 1024:
                     mem_usage_line = Text.assemble(
                         "Memory usage: ",
-                        ((spark_str, "dark_green")),
+                        ((spark_str, self.memory_color)),
                         (
                             f" (max: {(stats.max_footprint / 1024):6.2f}GB, growth rate: {growth_rate:3.0f}%)\n"
                         ),
@@ -366,7 +376,7 @@ class ScaleneOutput:
                     # Otherwise, use MB.
                     mem_usage_line = Text.assemble(
                         "Memory usage: ",
-                        ((spark_str, "dark_green")),
+                        ((spark_str, self.memory_color)),
                         (
                             f" (max: {stats.max_footprint:6.2f}MB, growth rate: {growth_rate:3.0f}%)\n"
                         ),
@@ -410,7 +420,7 @@ class ScaleneOutput:
             stats.output_stats(pid, python_alias_dir)
             return True
 
-        if len(report_files) == 0:
+        if not report_files:
             return False
 
         for fname in report_files:
@@ -420,17 +430,16 @@ class ScaleneOutput:
             fname_print = fname
             import re
 
-            result = re.match("<ipython-input-([0-9]+)-.*>", fname_print)
-            if result:
-                fname_print = Filename("[" + result.group(1) + "]")
+            if result := re.match("<ipython-input-([0-9]+)-.*>", fname_print):
+                fname_print = Filename(f"[{result.group(1)}]")
 
             # Print header.
-            if not stats.total_cpu_samples:
-                percent_cpu_time = 0
-            else:
-                percent_cpu_time = (
-                    100 * stats.cpu_samples[fname] / stats.total_cpu_samples
-                )
+            percent_cpu_time = (
+                (100 * stats.cpu_samples[fname] / stats.total_cpu_samples)
+                if stats.total_cpu_samples
+                else 0
+            )
+
             new_title = mem_usage_line + (
                 f"{fname_print}: % of time = {percent_cpu_time:6.2f} out of {stats.elapsed_time:6.2f}."
             )
@@ -471,8 +480,8 @@ class ScaleneOutput:
             )
             if self.gpu:
                 tbl.add_column(
-                    Markdown("––––––  \n_GPU_", style="yellow4"),
-                    style="yellow4",
+                    Markdown("––––––  \n_GPU_", style=self.gpu_color),
+                    style=self.gpu_color,
                     no_wrap=True,
                     width=6,
                 )
@@ -481,52 +490,47 @@ class ScaleneOutput:
 
             if profile_memory:
                 tbl.add_column(
-                    Markdown("Memory  \n_Python_", style="dark_green"),
-                    style="dark_green",
+                    Markdown("Memory  \n_Python_", style=self.memory_color),
+                    style=self.memory_color,
                     no_wrap=True,
                     width=7,
                 )
                 tbl.add_column(
-                    Markdown("––––––  \n_peak_", style="dark_green"),
-                    style="dark_green",
+                    Markdown("––––––  \n_peak_", style=self.memory_color),
+                    style=self.memory_color,
                     no_wrap=True,
                     width=6,
                 )
                 tbl.add_column(
                     Markdown(
-                        "–––––––––––  \n_timeline_/%", style="dark_green"
+                        "–––––––––––  \n_timeline_/%", style=self.memory_color
                     ),
-                    style="dark_green",
+                    style=self.memory_color,
                     no_wrap=True,
                     width=15,
                 )
                 tbl.add_column(
-                    Markdown("Copy  \n_(MB/s)_", style="yellow4"),
-                    style="yellow4",
+                    Markdown("Copy  \n_(MB/s)_", style=self.copy_volume_color),
+                    style=self.copy_volume_color,
                     no_wrap=True,
                     width=6,
                 )
                 other_columns_width = 75 + (6 if self.gpu else 0)
-                tbl.add_column(
-                    "\n" + fname_print,
-                    width=column_width - other_columns_width,
-                    no_wrap=True,
-                )
             else:
                 other_columns_width = 37 + (5 if self.gpu else 0)
-                tbl.add_column(
-                    "\n" + fname_print,
-                    width=column_width - other_columns_width,
-                    no_wrap=True,
-                )
-
+            tbl.add_column(
+                "\n" + fname_print,
+                width=column_width - other_columns_width,
+                no_wrap=True,
+            )
             # Print out the the profile for the source, line by line.
             if fname == "<BOGUS>":
                 continue
             if not fname:
                 continue
             # Print out the profile for the source, line by line.
-            with open(fname, "r", encoding="utf-8") as source_file:
+            full_fname = os.path.normpath(os.path.join(program_path, fname))
+            with open(full_fname, "r", encoding="utf-8") as source_file:
                 # We track whether we should put in ellipsis (for reduced profiles)
                 # or not.
                 did_print = True  # did we print a profile line last time?
@@ -601,11 +605,10 @@ class ScaleneOutput:
                         tbl.add_row("", "", "", "", "", "", "", "", "", txt)
                     else:
                         tbl.add_row("", "", "", "", "", "", "", "", txt)
+                elif self.gpu:
+                    tbl.add_row("", "", "", "", "", txt)
                 else:
-                    if self.gpu:
-                        tbl.add_row("", "", "", "", "", txt)
-                    else:
-                        tbl.add_row("", "", "", "", txt)
+                    tbl.add_row("", "", "", "", txt)
 
                 for fn_name in sorted(
                     fn_stats.cpu_samples_python,
@@ -643,8 +646,7 @@ class ScaleneOutput:
             avg_mallocs: Dict[LineNumber, float] = defaultdict(float)
             for line_no in stats.bytei_map[fname]:
                 n_malloc_mb = stats.memory_aggregate_footprint[fname][line_no]
-                count = stats.memory_malloc_count[fname][line_no]
-                if count:
+                if count := stats.memory_malloc_count[fname][line_no]:
                     avg_mallocs[line_no] = n_malloc_mb / count
                 else:
                     # Setting to n_malloc_mb addresses the edge case where this allocation is the last line executed.
@@ -699,11 +701,10 @@ class ScaleneOutput:
             if not self.output_file:
                 self.output_file = "/dev/stdout"
             console.save_html(self.output_file, clear=False)
+        elif self.output_file:
+            # Don't output styles to text file.
+            console.save_text(self.output_file, styles=False, clear=False)
         else:
-            if not self.output_file:
-                # No output file specified: write to stdout.
-                sys.stdout.write(console.export_text(styles=True))
-            else:
-                # Don't output styles to text file.
-                console.save_text(self.output_file, styles=False, clear=False)
+            # No output file specified: write to stdout.
+            sys.stdout.write(console.export_text(styles=True))
         return True
