@@ -229,7 +229,7 @@ class SampleHeap : public SuperHeap {
 
     writeCount(FreeSignal, sampleFree, nullptr, filename, lineno, bytei);
 #if !SCALENE_DISABLE_SIGNALS
-    raise(FreeSignal);  // was FreeSignal
+    raise(FreeSignal);
 #endif
     freeTriggered()++;
   }
@@ -281,7 +281,7 @@ class SampleHeap : public SuperHeap {
       _pythonCount = 1;  // prevent 0/0
     }
     snprintf_(
-        buf, SampleFile::MAX_BUFSIZE,
+        buf, sizeof(buf),
 #if defined(__APPLE__)
         "%c,%llu,%llu,%f,%d,%p,%s,%d,%d\n\n",
 #else
